/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <perlin.hpp>
#include <simplexnoise1234.hpp>
#include <v2.hpp>
#include <v3.hpp>
#include <v4.hpp>

#include <cmath>

namespace cpe
{

double perlin::noise(const double x,int octave,double persistency)
{

    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise1(x*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const v2& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise2(x.x()*frequency,x.y()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const v3& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise3(x.x()*frequency,x.y()*frequency,x.z()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

double perlin::noise(const v4& x,int octave,double persistency)
{
    double temp=0.0;
    double frequency=1.0;
    double current_persistency=1.0;
    for(int k=0;k<octave;k++)
    {

        temp += current_persistency*(0.5+0.5*snoise4(x.x()*frequency,x.y()*frequency,x.z()*frequency,x.w()*frequency));
        frequency *= 2.0;
        current_persistency *= persistency;
    }

    //normalize
    double epsilon=1e-5;
    if( std::abs(current_persistency-1)>epsilon )
        temp *= (1-persistency)/(1-current_persistency);

    return temp;
}

}
